#include "inotify-masks.h"
#include "inotifyutils.h"

#define _GNU_SOURCE
#include <strings.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <assert.h>



/**
 * Convert event from string form to integer form (as in inotify.h).
 *
 * @param    event    event in string form as defined in inotify.h without
 *                    leading IN_ prefix (e.g., MODIFY, ATTRIB)
 * @return            integer representing the mask specified by 'event'
 */
int str_to_event(char const * event)
{
	if ( 0 == strcasecmp(event, "ACCESS") )
		return IN_ACCESS;
	if ( 0 == strcasecmp(event, "MODIFY") )
		return IN_MODIFY;
	if ( 0 == strcasecmp(event, "ATTRIB") )
		return IN_ATTRIB;
	if ( 0 == strcasecmp(event, "CLOSE_WRITE") )
		return IN_CLOSE_WRITE;
	if ( 0 == strcasecmp(event, "CLOSE_NOWRITE") )
		return IN_CLOSE_NOWRITE;
	if ( 0 == strcasecmp(event, "OPEN") )
		return IN_OPEN;
	if ( 0 == strcasecmp(event, "MOVED_FROM") )
		return IN_MOVED_FROM;
	if ( 0 == strcasecmp(event, "MOVED_TO") )
		return IN_MOVED_TO;
	if ( 0 == strcasecmp(event, "CREATE") )
		return IN_CREATE;
	if ( 0 == strcasecmp(event, "DELETE") )
		return IN_DELETE;
	if ( 0 == strcasecmp(event, "DELETE_SELF") )
		return IN_DELETE_SELF;
	if ( 0 == strcasecmp(event, "UNMOUNT") )
		return IN_UNMOUNT;
	if ( 0 == strcasecmp(event, "Q_OVERFLOW") )
		return IN_Q_OVERFLOW;
	if ( 0 == strcasecmp(event, "IGNORED") )
		return IN_IGNORED;
	if ( 0 == strcasecmp(event, "CLOSE") )
		return IN_CLOSE;
	if ( 0 == strcasecmp(event, "MOVE") )
		return IN_MOVE;
	if ( 0 == strcasecmp(event, "ISDIR") )
		return IN_ISDIR;
	if ( 0 == strcasecmp(event, "ONESHOT") )
		return IN_ONESHOT;
	if ( 0 == strcasecmp(event, "ALL_EVENTS") )
		return IN_ALL_EVENTS;

	return -1;
}


/**
 * Convert event from integer form to string form (as in inotify.h).
 *
 * @param    event    OR'd event(s) in integer form as defined in inotify.h
 * @return            string representing the event(s)
 */
char * event_to_str(int events)
{
	char * ret = NULL;

	if ( (IN_ACCESS & events) != 0 )
		append(&ret, " ACCESS");
	if ( (IN_MODIFY & events) != 0 )
		append(&ret, " MODIFY");
	if ( (IN_ATTRIB & events) != 0 )
		append(&ret, " ATTRIB");
	if ( (IN_CLOSE_WRITE & events) != 0 )
		append(&ret, " CLOSE_WRITE");
	if ( (IN_CLOSE_NOWRITE & events) != 0 )
		append(&ret, " CLOSE_NOWRITE");
	if ( (IN_OPEN & events) != 0 )
		append(&ret, " OPEN");
	if ( (IN_MOVED_FROM & events) != 0 )
		append(&ret, " MOVED_FROM");
	if ( (IN_MOVED_TO & events) != 0 )
		append(&ret, " MOVED_TO");
	if ( (IN_CREATE & events) != 0 )
		append(&ret, " CREATE");
	if ( (IN_DELETE & events) != 0 )
		append(&ret, " DELETE");
	if ( (IN_DELETE_SELF & events) != 0 )
		append(&ret, " DELETE_SELF");
	if ( (IN_UNMOUNT & events) != 0 )
		append(&ret, " UNMOUNT");
	if ( (IN_Q_OVERFLOW & events) != 0 )
		append(&ret, " Q_OVERFLOW");
	if ( (IN_IGNORED & events) != 0 )
		append(&ret, " IGNORED");
	if ( (IN_CLOSE & events) != 0 )
		append(&ret, " CLOSE");
	if ( (IN_MOVE & events) != 0 )
		append(&ret, " MOVE");
	if ( (IN_ISDIR & events) != 0 )
		append(&ret, " ISDIR");
	if ( (IN_ONESHOT & events) != 0 )
		append(&ret, " ONESHOT");

	// Maybe we didn't match any... ?
	if ( !ret || (strlen(ret) < 1) )
	{
		if ( ret ) free(ret);
		if ( -1 == asprintf( &ret, " (unknown event(s); bitmask is %d)", events ) ) {
			assert( false );
		}
	}

	// Cut off the first space
	char * newret = strdup(&ret[1]);
	assert( newret );
	free(ret);
	return newret;
}


void append(char ** s1, char const * s2)
{
	int bytes;
	char * newstr;
	if ( !(*s1) ) {
		bytes = asprintf( s1, "%s", s2 );
	}
	else {
		bytes = asprintf( &newstr, "%s%s", *s1, s2 );
		free(*s1);
		*s1 = newstr;
	}
	assert( -1 != bytes );
}
